#include <M5Stack.h>
#include <SD.h>
#include "time.h"
#include "WifiWrapper.h"
#include "WeatherWarningYahoo.h"

#define SD_CHIPSELECT 4

const char *ntpServer =  "ntp.jst.mfeed.ad.jp";   //日本のNTPサーバー
const long gmtOffset = 9 * 3600;  // 時差（9時間）をセット
const int daylightOffset = 0;    // 夏時間はなし
bool initNTP = false;

#define TIMEOUT_COUNT 3

hw_timer_t *interruptGetTimer = NULL;
bool dispTimerEnable = false;
portMUX_TYPE mutex = portMUX_INITIALIZER_UNLOCKED;
volatile int intrGetCount = TIMEOUT_COUNT;  // 初回で必ず取得するようにする

//TimerIntr
void IRAM_ATTR getCountIntr()
{
  portENTER_CRITICAL_ISR(&mutex);
  intrGetCount++;
  portEXIT_CRITICAL_ISR(&mutex);
}

void setup() {
  M5.begin();
  M5.Lcd.setBrightness(120);
  wifiWrapper.setup();
  SD.begin(SD_CHIPSELECT);

  M5.Lcd.clearDisplay(WHITE);

  M5.Lcd.setCursor(0, 0);
  M5.Lcd.setTextColor(BLACK);

  //interrupt timer setting
  interruptGetTimer = timerBegin(0, 80, true);
  timerAttachInterrupt(interruptGetTimer, &getCountIntr, true);
  timerAlarmWrite(interruptGetTimer, 1000000, true);  // 1秒に一回感覚でタイマー割り込み発生
  timerAlarmEnable(interruptGetTimer);

}

void printLocalTime()
{
  struct tm timeinfo;
  if (!getLocalTime(&timeinfo)) {
    Serial.println("Failed to obtain time");
    return;
  }
  Serial.println(&timeinfo, "%Y %m %d %a %H:%M:%S");
}

void loop() {
  int count = 0;

  // まずは、NTPで時刻調整する
  if(!initNTP) {
    if (wifiWrapper.isConnect() != true) {
      Serial.printf("WiFi.status() not connect\n");
      // wait for WiFi connection
      delay(1000);
    }
    else {
      // NTPから時刻を取得してシステムタイムとする
      configTime(gmtOffset, daylightOffset, ntpServer);
      initNTP = true;
    }
    goto FUNC_END;
  }

  portENTER_CRITICAL_ISR(&mutex);
  count = intrGetCount;
  portEXIT_CRITICAL_ISR(&mutex);

  // ３秒に１回取得・表示する
  if(count >= TIMEOUT_COUNT) {
    weatherWarningYahoo.getWarning();
    printLocalTime();
    portENTER_CRITICAL_ISR(&mutex);
    intrGetCount = 0;
    portEXIT_CRITICAL_ISR(&mutex);
  }


 FUNC_END:
  M5.update();
}