#include "WifiWrapper.h"
#include <HTTPClient.h>
#include <tinyxml2.h>
#include "WeatherWarningYahoo.h"

const char* host = "https://rss-weather.yahoo.co.jp/rss/warn/";
/* pref : 下記の情報から選択して設定する */
/* areaName : https://typhoon.yahoo.co.jp/weather/jp/warn/ から取得したい地域を選択します */
const char* pref = "13.xml";
const char* areaName = "新宿区";

/***********************/
/* 都道府県（地域）情報  */
/***********************/
/*
    【北海道】 
         【道北】  pref : "1a.xml"
         【道央】  pref : "1b.xml"
         【道東】  pref : "1c.xml"
         【道南】  pref : "1d.xml"

    【青森県】  pref : "2.xml"
    【岩手県】  pref : "3.xml"
    【宮城県】  pref : "4.xml"
    【秋田県】  pref : "5.xml"
    【山形県】  pref : "6.xml"
    【福島県】  pref : "7.xml"
    【茨城県】  pref : "8.xml"
    【栃木県】  pref : "9.xml"
    【群馬県】  pref : "10.xml"
    【埼玉県】  pref : "11.xml"
    【千葉県】  pref : "12.xml"
    【東京都】  pref : "13.xml"
    【神奈川県】  pref : "14.xml"
    【新潟県】  pref : "15.xml"
    【富山県】  pref : "16.xml"
    【石川県】  pref : "17.xml"
    【福井県】  pref : "18.xml"
    【山梨県】  pref : "19.xml"
    【長野県】  pref : "20.xml"
    【岐阜県】  pref : "21.xml"
    【静岡県】  pref : "22.xml"
    【愛知県】  pref : "23.xml"
    【三重県】  pref : "24.xml"
    【滋賀県】  pref : "25.xml"
    【京都府】  pref : "26.xml"
    【大阪府】  pref : "27.xml"
    【兵庫県】  pref : "28.xml"
    【奈良県】  pref : "29.xml"
    【和歌山県】  pref : "30.xml"
    【鳥取県】  pref : "31.xml"
    【島根県】  pref : "32.xml"
    【岡山県】  pref : "33.xml"
    【広島県】  pref : "34.xml"
    【山口県】  pref : "35.xml"
    【徳島県】  pref : "36.xml"
    【香川県】  pref : "37.xml"
    【愛媛県】  pref : "38.xml"
    【高知県】  pref : "39.xml"
    【福岡県】  pref : "40.xml"
    【佐賀県】  pref : "41.xml"
    【長崎県】  pref : "42.xml"
    【熊本県】  pref : "43.xml"
    【大分県】  pref : "44.xml"
    【宮崎県】  pref : "45.xml"
    【鹿児島県】  pref : "46.xml"
    【沖縄県】  pref : "47.xml"
*/

const char* dirName = "/warn/";
const char* noneFname = "nothing.png";

const char* warning_prefix_name[WARN_MAX] = {
    "wa_",
    "al_",
    "sp_"
};

const char* warning_fname[WARN_KIND_MAX] = {
    "heavy_rain.png",     /* 大雨（注意報・警報・特別警報） */
    "flood.png",          /* 洪水（注意報・警報） */
    "strong_wind.png",    /* 強風（注意報）、暴風（警報・特別警報） */
    "wind_and_snow.png",  /* 風雪（注意報）、暴風雪（警報・特別警報） */
    "heavy_snow.png",     /* 大雪（注意報・警報・特別警報） */
    "waves.png",          /* 波浪（注意報・警報・特別警報） */
    "high_tide.png",      /* 高潮（注意報・警報・特別警報） */
    "thunder.png",        /* 雷（注意報） */
    "melting_snow.png",   /* 融雪（注意報） */
    "thick_fog.png",      /* 濃霧（注意報） */
    "dry.png",            /* 乾燥（注意報） */
    "avalanche.png",      /* なだれ（注意報） */
    "low_temp.png",       /* 低温（注意報） */
    "frost.png",          /* 霜（注意報） */
    "icing.png",          /* 着氷（注意報） */
    "snow_landing.png",   /* 着雪（注意報） */
    ""
};

using namespace tinyxml2;

WeatherWarningYahoo::WeatherWarningYahoo()
{
    // URLを作成する
    m_URL = host;
    m_URL += pref;

    m_LinkURL = "";
    m_SimpleLinkURL = "";

    m_first = true;
}

bool
WeatherWarningYahoo::getLinkURL()
{
    bool ret = true;

    // ネットワーク接続状態確認
    if (wifiWrapper.isConnect() != true) {
        Serial.printf("[WeatherJMA]WiFi.status() not connect\n");
        return false;
    }

    HTTPClient http;
    XMLDocument doc;

    http.begin(m_URL);

    int httpCode = http.GET();
    if (httpCode > 0)
    {
        if(doc.Parse(http.getString().c_str()) != XML_SUCCESS){
            Serial.println("Error parsing LinkURL"); 
            ret = false;
        }
        else {
            XMLElement* linkElem = doc.FirstChildElement("rss")->FirstChildElement("channel")->FirstChildElement("link");
            m_LinkURL = linkElem->GetText();
            Serial.println(m_LinkURL); 
        }
    }
    else
    {
        Serial.println("Error on HTTP request");
        ret = false;
    }
    http.end(); //リソースを解放

    return ret;
}

bool
WeatherWarningYahoo::getSimpleLinkURL()
{
    bool ret = true;

    // ネットワーク接続状態確認
    if (wifiWrapper.isConnect() != true) {
        Serial.printf("[WeatherJMA]WiFi.status() not connect\n");
        return false;
    }

    HTTPClient http;
    XMLDocument doc;

    http.begin(m_LinkURL);
    int httpCode = http.GET();
    if (httpCode > 0)
    {
        if(doc.Parse(http.getString().c_str()) != XML_SUCCESS){
            Serial.println("Error parsing simpleLink"); 
            ret = false;
        }
        else {
            XMLElement* linkElem = doc.FirstChildElement("p")->FirstChildElement("a");
            m_SimpleLinkURL = linkElem->Attribute("href");
        }
    }
    else
    {
        Serial.println("Error on HTTP request");
        ret = false;
    }
    http.end(); //リソースを解放

    return ret;
}

WARN_TYPE_DEF
WeatherWarningYahoo::getType(const char* str)
{
    String tmp_str = str;
    // Serial.println(tmp_str);

    if(tmp_str == String("特別警報")) {
        return SPECIAL_ALARM;
    }
    if(tmp_str == String("警報")) {
        return ALARM;
    }
    if(tmp_str == String("注意報")) {
        return WARNING;
    }
    // 異常
    return WARN_MAX;
}

WARN_KIND_DEF
WeatherWarningYahoo::getKind(const char* str)
{
    String tmp_str = str;
    // Serial.println(tmp_str);

    if(tmp_str == String("大雨")) {
        return HEAVY_RAIN;
    }
    if(tmp_str == String("洪水")) {
        return FLOOD;
    }
    if(tmp_str == String("風雪") || tmp_str == String("暴風雪")) {
        return WIND_AND_SNOW;
    }
    if(tmp_str == String("強風") || tmp_str == String("暴風")) {
        return STRONG_WIND;
    }
    if(tmp_str == String("大雪")) {
        return HEAVY_SNOW;
    }
    if(tmp_str == String("波浪")) {
        return WAVES;
    }
    if(tmp_str == String("高潮")) {
        return HIGH_TIDE;
    }
    if(tmp_str == String("雷")) {
        return THUNDER;
    }
    if(tmp_str == String("融雪")) {
        return MELTING_SNOW;
    }
    if(tmp_str == String("濃霧")) {
        return THICK_FOG;
    }
    if(tmp_str == String("乾燥")) {
        return DRY;
    }
    if(tmp_str == String("なだれ")) {
        return AVALANCHE;
    }
    if(tmp_str == String("低温")) {
        return LOW_TEMP;
    }
    if(tmp_str == String("霜")) {
        return FROST;
    }
    if(tmp_str == String("着氷")) {
        return ICING;
    }
    if(tmp_str == String("着雪")) {
        return SNOW_LANDING;
    }
    if(tmp_str == String("発表なし")) {
        return WARN_NONE;
    }
    // 異常
    return WARN_KIND_MAX;
}

bool
WeatherWarningYahoo::getWarnAlarm()
{
    bool ret = true;
    String href = "";
    std::vector<WarnTable> warnTable_tmp;
    uint8_t tmp_count = 0;
    bool same = true;

    // ネットワーク接続状態確認
    if (wifiWrapper.isConnect() != true) {
        Serial.printf("[WeatherJMA]WiFi.status() not connect\n");
        return false;
    }

    HTTPClient http;
    XMLDocument doc;

    http.begin(m_SimpleLinkURL);
    int httpCode = http.GET();
    if (httpCode > 0)
    {
        int area = -1, from = -1, to = -1;

        String tmp_str = http.getString();
        area = tmp_str.indexOf(areaName);
        if(area == -1) {
            Serial.println("Nothing AreaName");
            ret = false;
            goto FUNC_END;
        }
        from = tmp_str.indexOf("<td class", area);
        if(from == -1) {
            Serial.println("Nothing Start of Warn Msg");
            ret = false;
            goto FUNC_END;
        }
        to =  tmp_str.indexOf("</tr>", from);
        if(to == -1) {
            Serial.println("Nothing End of Warn Msg");
            ret = false;
            goto FUNC_END;
        }
        String warn_str = tmp_str.substring(from, to);
        warn_str += "\0";

        /* XML文字列置き換え（念のため） */
        warn_str.replace( "&amp;", "＆" );      /* 半角& -> 全角に置き換える */
        warn_str.replace( "&#039;", "\'" );     /* 半角アポストロフィー */
        warn_str.replace( "&#39;", "\'" );      /* 半角アポストロフィー */
        warn_str.replace( "&apos;", "\'" );     /* 半角アポストロフィー */
        warn_str.replace( "&quot;", "\"" );     /* ダブルクォーテーション */

        Serial.println(warn_str);

        if(doc.Parse(warn_str.c_str()) != XML_SUCCESS){
            Serial.println("Error parsing warning"); 
            ret = false;
            goto FUNC_END;
        }
        else {
            XMLElement *pTdElem = doc.FirstChildElement("td");
            if(!pTdElem) {
                Serial.println("Nothing pTdElem"); 
                ret = false;
                goto FUNC_END;
            }

            XMLElement *pClassSpanElem = pTdElem->FirstChildElement("span");
            if(!pClassSpanElem) {
                Serial.println("Nothing Class Elem"); 
                ret = false;
                goto FUNC_END;
            }
            for( ; pClassSpanElem; pClassSpanElem = pClassSpanElem->NextSiblingElement()) {
                XMLElement *pWarnSpanElem = pClassSpanElem->FirstChildElement("span");
                if(!pWarnSpanElem) {
                    Serial.println("Nothing Warn Span Elem"); 
                    ret = false;
                    goto FUNC_END;
                }
                XMLElement *pSmallElem = pWarnSpanElem->FirstChildElement("small");
                if(!pSmallElem) {
                    Serial.println("Nothing Small Span Elem"); 
                    // 発表が無い場合は、"small"エレメントは無いので継続
                }
                WARN_KIND_DEF kind = getKind(pWarnSpanElem->GetText());
                if(kind != WARN_NONE) {
                    WARN_TYPE_DEF type = getType(pSmallElem->GetText());
                    //Serial.printf("type = %d kind = %d \n", type, kind);
                    if(type != WARN_MAX && kind != WARN_KIND_MAX) {
                        WarnTable tbl_tmp = {type, kind};
                        warnTable_tmp.push_back(tbl_tmp);
                        tmp_count++;
                    }
                }
                /* 4個だけ表示 */
                if(tmp_count >= 4) {
                    break;
                }
            }
            if(m_first) {
                m_first = false;
                /* 初回は異なるものとして扱う */
                same = false;
            }
            else {
                if(m_WarnTable.size() == warnTable_tmp.size()) {
                    for(auto itr_tmp = warnTable_tmp.begin(); itr_tmp < warnTable_tmp.end(); itr_tmp++ ) {
                        auto f = std::find(m_WarnTable.begin(), m_WarnTable.end(), *itr_tmp);
                        if(f == m_WarnTable.end()) {
                            /* 見つからないものがあった => 値が異なる */
                            same = false;
                            //Serial.printf("new kind = %d, type = %d\n", itr_tmp->kind, itr_tmp->type);
                            break;
                        }
                    }
                }
                else {
                    same = false;
                }
            }
            if(same) {
                Serial.println("Same Content");
                ret = false;
            }
            else {
                m_WarnTable.clear();
                m_WarnTable = warnTable_tmp;
                m_WarnTable.shrink_to_fit();
            }
        }
    }
    else
    {
        Serial.println("Error on HTTP request");
        ret = false;
    }

  FUNC_END:
    http.end(); //リソースを解放

    return ret;
}

void
WeatherWarningYahoo::drawWarnAlarm()
{
    int i = 0;
    String fname = "";
    uint16_t x[4] = {0}, y[4] = {0};

    int len = m_WarnTable.size();

    /* 警告が無い場合 */
    if( len == 0 ) {
        M5.Lcd.clearDisplay(WHITE);
        fname = dirName;
        fname += noneFname;
        M5.Lcd.drawPngFile(SD, fname.c_str(), x[0], y[0]);
        return;
    }

    M5.Lcd.clearDisplay(WHITE);
    if( len == 1 ) {
        x[0] = 88;
        y[0] = 70;
    }
    else if( len == 2 ) {
        x[0] = 10;
        x[1] = 165;
        y[0] = y[1] = 70;
    }
    else {
        x[0] = x[2] = 10;
        x[1] = x[3] = 165;
        y[0] = y[1] = 15;
        y[2] = y[3] = 125;
    }

    for(auto itr = m_WarnTable.begin(); itr < m_WarnTable.end(); itr++ ) {
        fname = dirName;
        fname += warning_prefix_name[itr->type];
        fname += warning_fname[itr->kind];
        M5.Lcd.drawPngFile(SD, fname.c_str(), x[i], y[i]);
        i++;
    }

}

void
WeatherWarningYahoo::getWarning()
{
    /* LinkURLを取得 */
    if(!getLinkURL()) {
        return;
    }
    delay(1);

    /* SimpleLinkURLを取得(RSSにはリンク用のURLが載っている) */
    if(!getSimpleLinkURL()) {
        return;
    }
    delay(1);

    /* 警告情報を取得 */
    if(!getWarnAlarm()) {
        return;
    }

    /* 警告表示 */
    drawWarnAlarm();
}

WeatherWarningYahoo weatherWarningYahoo;