#ifndef WEATHER_WARNING_YAHOO_H
#define WEATHER_WARNING_YAHOO_H

#include <M5Stack.h>
#include "time.h"

typedef enum {
    WARNING,        /* 注意報 */
    ALARM,          /* 警報 */
    SPECIAL_ALARM,  /* 特別警報 */
    WARN_MAX        /* MAX */
} WARN_TYPE_DEF;

typedef enum {
    HEAVY_RAIN,     /* 大雨（注意報・警報・特別警報） */
    FLOOD,          /* 洪水（注意報・警報） */
    STRONG_WIND,    /* 強風（注意報）、暴風（警報・特別警報） */
    WIND_AND_SNOW,  /* 風雪（注意報）、暴風雪（警報・特別警報） */
    HEAVY_SNOW,     /* 大雪（注意報・警報・特別警報） */
    WAVES,          /* 波浪（注意報・警報・特別警報） */
    HIGH_TIDE,      /* 高潮（注意報・警報・特別警報） */
    THUNDER,        /* 雷（注意報） */
    MELTING_SNOW,   /* 融雪（注意報） */
    THICK_FOG,      /* 濃霧（注意報） */
    DRY,            /* 乾燥（注意報） */
    AVALANCHE,      /* なだれ（注意報） */
    LOW_TEMP,       /* 低温（注意報） */
    FROST,          /* 霜（注意報） */
    ICING,          /* 着氷（注意報） */
    SNOW_LANDING,   /* 着雪（注意報） */
    WARN_NONE,      /* 警報なし */
    WARN_KIND_MAX   /* MAX */
} WARN_KIND_DEF;

typedef struct _WarnTable {
  int type;
  int kind;

  bool operator ==(const _WarnTable &l) {
    return( (l.type == type) && (l.kind == kind) );
  }
} WarnTable;

class WeatherWarningYahoo {
private:
  String m_URL;
  String m_LinkURL;
  String m_SimpleLinkURL;
  std::vector<WarnTable> m_WarnTable;
  bool m_first;

  bool getLinkURL();
  bool getSimpleLinkURL();
  WARN_TYPE_DEF getType(const char* str);
  WARN_KIND_DEF getKind(const char* str);
  bool getWarnAlarm();
  void drawWarnAlarm();

public:
  WeatherWarningYahoo();

  void getWarning();
};

extern WeatherWarningYahoo weatherWarningYahoo;

#endif // WEATHER_WARNING_YAHOO_H
